function offspring_chromosome = genetic_operator(mate_pool_chromosome, pool_size, n, lu, pcrossover, pmutation, di, dim)
% Initialize
offspring_chromosome = zeros(pool_size, n);

i = 1;
while i <= pool_size
    
    parent1 = mate_pool_chromosome(i , :);
    parent2 = mate_pool_chromosome(i+1 , :);
    
    [ child1, child2 ] = crossover(parent1, parent2, n, lu, pcrossover, di);
    
    offspring_chromosome(i, :) = child1;
    offspring_chromosome(i+1, :) = child2;
    
    i = i + 2;
    
end

offspring_chromosome = mutation(offspring_chromosome, pool_size, n, lu, pmutation, dim);

end

function [child1, child2] = crossover(parent1, parent2, n, lu, pcrossover, di)

parent_1 = parent1(1 : n);
parent_2 = parent2(1 : n);

if rand <= pcrossover
    
    child1 = zeros(1, n);
    child2 = zeros(1, n);
    
    for i = 1:n
        
        if rand <= 0.5
            
            if abs(parent_1(i)-parent_2(i)) > 1e-14
                
                if parent_2(i) > parent_1(i)
                    y2 = parent_2(i);
                    y1 = parent_1(i);
                else
                    y2 = parent_1(i);
                    y1 = parent_2(i);
                end
                
                y_low=lu(1, i);
                y_upper=lu(2, i);
                
                beta = 1.0 + (2.0*(y1-y_low)/(y2-y1));
                
                expp = -(di + 1.0);
                alpha = 2.0 - power(beta, expp);
                
                rnd=rand;
                if rnd <= 1.0/alpha
                    
                    alpha = alpha*rnd;
                    expp = 1.0/(di+1.0);
                    betaq = power(alpha,expp);
                    
                else
                    
                    alpha = alpha*rnd;
                    alpha = 1.0/(2.0-alpha);
                    expp = 1.0/(di+1.0);
                    betaq = power(alpha,expp);
                    
                end
                
                temp_child1 = 0.5*((y1+y2) - betaq*(y2-y1));
                
                beta = 1.0 + (2.0*(y_upper-y2)/(y2-y1));
                
                expp = -(di + 1.0);
                alpha = 2.0 - power(beta,expp);
                
                rnd=rand;
                if rnd <= 1.0/alpha
                    
                    alpha = alpha*rnd;
                    expp = 1.0/(di+1.0);
                    betaq = power(alpha,expp);
                    
                else
                    
                    alpha = alpha*rnd;
                    alpha = 1.0/(2.0-alpha);
                    expp = 1.0/(di+1.0);
                    betaq = power(alpha,expp);
                    
                end
                
                temp_child2 = 0.5*((y1+y2) + betaq*(y2-y1));
                
                if temp_child1 < y_low
                    temp_child1 = y_low;
                end
                if temp_child1 > y_upper
                    temp_child1 = y_upper;
                end
                
                if temp_child2 < y_low
                    temp_child2 = y_low;
                end
                if temp_child2 > y_upper
                    temp_child2 = y_upper;
                end
                
                if rand <= 0.5
                    child1(i) = temp_child2;
                    child2(i) = temp_child1;
                else
                    child1(i) = temp_child1;
                    child2(i) = temp_child2;
                end
                
            else
                
                child1(i) = parent_1(i);
                child2(i) = parent_2(i);
                
            end
            
        else
            
            child1(i) = parent_1(i);
            child2(i) = parent_2(i);
            
        end
        
    end
    
else
    
    child1 = parent_1;
    child2 = parent_2;
    
end
end

function offspring_chromosome = mutation(offspring, popsize, n, lu, pmutation, dim)

offspring_chromosome = offspring;

for i = 1 : popsize
    
    for j = 1 : n
        
        if rand <= pmutation
            
            y = offspring(i, j);
            y_low = lu(1, j);
            y_upper = lu(2, j);
            
            delta1 = (y - y_low)/(y_upper - y_low);
            delta2 = (y_upper - y)/(y_upper - y_low);
            
            rnd = rand;
            
            indi = 1.0/(dim + 1.0);
            
            if rnd <= 0.5
                
                xy = 1.0 - delta1;
                val = 2.0*rnd + (1.0 - 2.0*rnd) * power(xy, (dim+1));
                deltaq =  power(val, indi) - 1.0;
                
            else
                
                xy = 1.0 - delta2;
                val = 2.0*(1.0-rnd) + 2.0*(rnd-0.5) * power(xy,(dim+1));
                deltaq = 1.0 - power(val,indi);
                
            end
            
            y = y + deltaq * (y_upper - y_low);
            
            if y < y_low
                y = y_low;
            end
            
            if y > y_upper
                y = y_upper;
            end
            
            offspring_chromosome(i, j) = y;
            
        end
        
    end
    
end
end
